package com.example.soulfood_assignment2;

import android.util.Log;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;

public class databaseInterface {
    public enum Cuisine {
        chinese,
        thai,
        indian,
        japanese,
        italian,
        mexican,
        french,
        spanish,
        english,
        american,
        any
    }
    public enum Vibe {
        chill,
        happy,
        energetic,
        calming,
        any
    }
    public enum Time {
        breakfast,
        lunch,
        dinner,
        any
    }

    /*
     * one string input - should still be in 123 format though
     * * returns array of spotify ids
     */
    public ArrayList<String> getPlaylistByID(String id) {
        String songids = getPlaylistSongIDsByID(id);

        ArrayList<String> ids = parseOutSpaces(songids);
        ArrayList<String> spotifys = new ArrayList<>();

        for (String indv: ids) {
            String spotty = getSongByID(indv);
            spotifys.add(spotty);
        }

        return spotifys;
    }

    /*
     * one string input - should still be in 123 format though
     * gets a Spotify id from the database
     */
    public String getSongByID(String id) {
        HashMap<String, String> params = new HashMap<>();
        params.put("id", id);

        String url = "https://cs2s.yorkdc.net/~jade.woodward/mobile-app-development/getsongbyid.php";
        return getFromDatabase(url, params);
    }

    /*
     * three enum inputs based on the database fields
     * returns an array of song ids that meet the criteria
     */
    public ArrayList<String> getPlaylistByAttributes(Cuisine c, Vibe v, Time t) {
        HashMap<String, String> params = new HashMap<>();

        String cuisine = c.name(), vibe = v.name(), time = t.name();
        if (cuisine.equals("any")) {
            cuisine = "%";
        } if (vibe.equals("any")) {
            vibe = "%";
        } if (time.equals("any")) {
            time = "%";
        }

        params.put("cuisine", cuisine);
        params.put("vibe", vibe);
        params.put("time", time);

        String url = "https://cs2s.yorkdc.net/~jade.woodward/mobile-app-development/getsongbyatributes.php";
        String songIDs =  getFromDatabase(url, params);

        ArrayList<String> ids = parseOutSpaces(songIDs);

        return ids;
    }

    /*
     * takes in an array list of spotify ids and a title for the playlist
     * makes a new playlist in the database
     * adds the songs to the playlist
     * returns true if the playlist was made correctly
     * returns false if something went wrong
     */
    public String savePlaylist(String title, ArrayList<String> spotifyIDs){
        HashMap<String, String> params = new HashMap<>();
        params.put("title", title);

        String url = "https://cs2s.yorkdc.net/~jade.woodward/mobile-app-development/newPlaylist.php";
        String playlistid = getFromDatabase(url, params);

        String url2 = "https://cs2s.yorkdc.net/~jade.woodward/mobile-app-development/getsongbyspotify.php";
        String url3 = "https://cs2s.yorkdc.net/~jade.woodward/mobile-app-development/addplaylistsongbyids.php";

        for (String s: spotifyIDs) {
            HashMap<String, String> param2 = new HashMap<>();
            param2.put("id", s);

            String id = getFromDatabase(url2, param2);

            HashMap<String, String> param3 = new HashMap<>();
            param3.put("song", id);
            param3.put("playlist", playlistid);

            if(!getFromDatabase(url3, param3).equals("added a song to playlist")){
                return "Failed to save to database";
            }
        }
        return playlistid;
    }

    /*
     * one string input - should still be in 123 format though
     * returns the three attributes for that song
     * in the order cuisine, vibe, time
     */
    public ArrayList<String> getAttributesBySongID(String id) {
        HashMap<String, String> params = new HashMap<>();
        params.put("id", id);

        String attribute = getFromDatabase("https://cs2s.yorkdc.net/~jade.woodward/mobile-app-development/getattributesbysongid.php", params);
        ArrayList<String> attributes = parseOutSpaces(attribute);

        return attributes;
    }

    /*
     * gets the title od a song from the database by the database song id
     */
    public String getSongTitle(String id){
        String url = "https://cs2s.yorkdc.net/~jade.woodward/mobile-app-development/getsongtitlebyid.php";

        HashMap<String, String> params = new HashMap<>();
        params.put("id", id);

        return getFromDatabase(url, params);
    }

    /*
     * one string input - should still be in 123 format though
     * gets a list of song ids from the database
     */
    private String getPlaylistSongIDsByID(String id) {
        HashMap<String, String> params = new HashMap<>();
        params.put("id", id);

        String url = "https://cs2s.yorkdc.net/~jade.woodward/mobile-app-development/getplaylistbyid.php";
        return getFromDatabase(url, params);
    }

    /*
     * base to fetch from the database
     */
    private String getFromDatabase(String url, HashMap <String, String> params) {
        StringBuilder sbParams = new StringBuilder();
        int i = 0;
        for (String key : params.keySet()) {
            try {
                if (i != 0){
                    sbParams.append("&");
                }
                sbParams.append(key).append("=").append(URLEncoder.encode(params.get(key), "UTF-8"));

            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            i++;
        }

        String spotify = "failed to connect to database";

        try {
            URL urlObj = new URL(url);
            HttpURLConnection conn = (HttpURLConnection) urlObj.openConnection();

            conn.setDoOutput(true);
            conn.setRequestMethod("POST");
            conn.setRequestProperty("Accept-Charset", "UTF-8");
            conn.setReadTimeout(10000);
            conn.setConnectTimeout(15000);
            conn.connect();

            String paramsString = sbParams.toString();

            DataOutputStream wr = new DataOutputStream(conn.getOutputStream());
            wr.writeBytes(paramsString);
            wr.flush();
            wr.close();

            try {
                InputStream is = conn.getInputStream();
                BufferedInputStream in = new BufferedInputStream(is);

                BufferedReader reader = new BufferedReader(new InputStreamReader(in));
                StringBuilder result = new StringBuilder();
                String line;

                while ((line = reader.readLine()) != null) {
                    result.append(line);
                }

                spotify = result.toString();

            } catch (IOException e) {
                e.printStackTrace();
            }

            if (conn != null) {
                conn.disconnect();
            }
        } catch (IOException e) {
            e.printStackTrace();
        }

        return spotify;
    }


    /*
     * splits a string into words by splitting at spaces.
     */
    private ArrayList<String> parseOutSpaces(String original) {
        ArrayList<String> results = new ArrayList<>();

        for(String id : original.split(" ")) {
            results.add(id);
        }

        return results;
    }
}
